@@ -4,7 +4,7 @@
  * Authors		: Patrick Lecoanet.
  * Creation date	: Fri Mar 25 15:32:17 1994
  *
- * $Id: Curve.c,v 1.50 2004/04/30 14:16:27 lecoanet Exp $
+ * $Id: Curve.c,v 1.43 2003/10/02 07:43:04 lecoanet Exp $
  */
 
 /*
@@ -39,7 +39,7 @@
 
 #include <ctype.h>
 
-static const char rcsid[] = "$Id: Curve.c,v 1.50 2004/04/30 14:16:27 lecoanet Exp $";
+static const char rcsid[] = "$Id: Curve.c,v 1.43 2003/10/02 07:43:04 lecoanet Exp $";
 static const char compile_id[]="$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";
 
 
@@ -490,29 +490,150 @@ Query(ZnItem		item,
       int		argc __unused,
       Tcl_Obj *CONST	argv[])
 {
-  if (ZnQueryAttribute(item->wi->interp, item, cv_attrs, argv[0]) == TCL_ERROR) {
+  if (ZnQueryAttribute(item->wi, item, cv_attrs, argv[0]) == TCL_ERROR) {
     return TCL_ERROR;
   }
 
   return TCL_OK;
 }
 
+
+static void
+CurveTessBegin(GLenum	type,
+	       void	*data)
+{
+  CurveItem	cv = data;
+  ZnWInfo	*wi = ((ZnItem) data)->wi;
+
+  ZnListEmpty(wi->work_pts);
+  wi->tess_type = type;
+  if (type == GL_LINE_LOOP) {
+    cv->outlines.num_contours++;
+    cv->outlines.contours = ZnRealloc(cv->outlines.contours,
+				      cv->outlines.num_contours * sizeof(ZnContour));
+  }
+  else {
+    cv->tristrip.num_strips++;
+    cv->tristrip.strips = ZnRealloc(cv->tristrip.strips,
+				    cv->tristrip.num_strips * sizeof(ZnStrip));
+    cv->tristrip.strips[cv->tristrip.num_strips-1].fan = (type==GL_TRIANGLE_FAN);
+  }
+  /*printf("Dbut de fragment de type: %s\n",
+	 (type == GL_TRIANGLE_FAN) ? "FAN" : 
+	 (type == GL_TRIANGLE_STRIP) ? "STRIP" :
+	 (type == GL_TRIANGLES) ? "TRIANGLES" :
+	 (type == GL_LINE_LOOP) ? "LINE LOOP" : "");*/
+}
+static void
+CurveTessVertex(void	*vertex_data,
+		void	*data)
+{
+  CurveItem	cv = data;
+  ZnWInfo	*wi = ((ZnItem) data)->wi;
+  ZnPoint	p;
+  int		size;
+
+  p.x = ((GLdouble *) vertex_data)[0];
+  p.y = ((GLdouble *) vertex_data)[1];
+  /*printf("Sommet en %g %g\n", p.x, p.y);*/
+  size = ZnListSize(wi->work_pts);
+  if ((wi->tess_type == GL_TRIANGLES) && (size == 3)) {
+    cv->tristrip.strips[cv->tristrip.num_strips-1].num_points = size;
+    cv->tristrip.strips[cv->tristrip.num_strips-1].points = ZnMalloc(size * sizeof(ZnPoint));
+    memcpy(cv->tristrip.strips[cv->tristrip.num_strips-1].points,
+	   ZnListArray(wi->work_pts), size * sizeof(ZnPoint));
+    /*printf("Fin de fragment intermediaire %d, num points: %d\n", cv->tristrip.num_strips-1, size);*/
+    /* Allocate a new fragment */
+    ZnListEmpty(wi->work_pts);
+    cv->tristrip.num_strips++;
+    cv->tristrip.strips = ZnRealloc(cv->tristrip.strips,
+				    cv->tristrip.num_strips * sizeof(ZnStrip));
+    cv->tristrip.strips[cv->tristrip.num_strips-1].fan = False;
+  }
+  ZnListAdd(wi->work_pts, &p, ZnListTail);
+}
+static void
+CurveTessEnd(void	*data)
+{
+  CurveItem	cv = data;
+  ZnWInfo	*wi = ((ZnItem) data)->wi;
+  unsigned int	size = ZnListSize(wi->work_pts);
+  unsigned int	num;
+
+  if (wi->tess_type == GL_LINE_LOOP) {
+    /* Add the last point to close the outline */
+    size++;
+    num = cv->outlines.num_contours;
+    cv->outlines.contours[num-1].num_points = size;
+    cv->outlines.contours[num-1].points = ZnMalloc(size * sizeof(ZnPoint));
+    memcpy(cv->outlines.contours[num-1].points,
+	   ZnListArray(wi->work_pts), size * sizeof(ZnPoint));
+    cv->outlines.contours[num-1].points[size-1] = cv->outlines.contours[num-1].points[0];
+    cv->outlines.contours[num-1].cw = !ZnTestCCW(cv->outlines.contours[num-1].points, size);
+  }
+  else {
+    num = cv->tristrip.num_strips;
+    cv->tristrip.strips[num-1].num_points = size;
+    cv->tristrip.strips[num-1].points = ZnMalloc(size * sizeof(ZnPoint));
+    memcpy(cv->tristrip.strips[num-1].points,
+	   ZnListArray(wi->work_pts), size * sizeof(ZnPoint));
+  }
+  /*printf("Fin de fragment %d, num points: %d\n", num, size);*/
+}
+static void
+CurveTessCombine(GLdouble	coords[3],
+		 void		*vertex_data[4] __unused,
+		 GLfloat	weight[4] __unused,
+		 void		**out_data,
+		 void		*data)
+{
+  ZnWInfo	*wi = ((ZnItem) data)->wi;
+  ZnCombineData	*cdata;
+  
+  cdata = ZnMalloc(sizeof(ZnCombineData));
+  cdata->v[0] = coords[0];
+  cdata->v[1] = coords[1];
+  cdata->next = wi->tess_combine_list;
+  wi->tess_combine_list = cdata;
+  *out_data = &cdata->v;
+  /*printf("Cration d'un nouveau sommet en %g %g\n",
+    cdata->v[0], cdata->v[1]);*/
+}
+static void
+CurveTessError(GLenum	errno,
+	       void	*data __unused)
+{
+  fprintf(stderr, "Tesselation error in curve item: %d\n", errno);
+}
+
 static void
 UpdateTristrip(CurveItem	cv,
 	       ZnPoly		*poly,
 	       ZnBool		revert)
 {
+  ZnWInfo	*wi = ((ZnItem) cv)->wi;
   ZnCombineData	*cdata, *cnext;
   GLdouble	v[3];
   unsigned int	j, k;
   int		i;
 
   /*printf("UpdateTristrips sur %d\n", ((ZnItem) cv)->id);*/
-  gluTessProperty(ZnTesselator.tess, GLU_TESS_WINDING_RULE, (GLdouble) cv->fill_rule);
+  gluTessCallback(wi->tess, GLU_TESS_BEGIN_DATA,
+		  (_GLUfuncptr) CurveTessBegin);
+  gluTessCallback(wi->tess, GLU_TESS_VERTEX_DATA,
+		  (_GLUfuncptr) CurveTessVertex);
+  gluTessCallback(wi->tess, GLU_TESS_END_DATA,
+		  (_GLUfuncptr) CurveTessEnd);
+  gluTessCallback(wi->tess, GLU_TESS_COMBINE_DATA,
+		  (_GLUfuncptr) CurveTessCombine);
+  gluTessCallback(wi->tess, GLU_TESS_ERROR_DATA,
+		  (_GLUfuncptr) CurveTessError);
+  gluTessProperty(wi->tess, GLU_TESS_WINDING_RULE, (GLdouble) cv->fill_rule);
+  gluTessNormal(wi->tess, 0.0, 0.0, -1.0);
 
   if (cv->tristrip.num_strips == 0) {
-    gluTessProperty(ZnTesselator.tess, GLU_TESS_BOUNDARY_ONLY, (GLdouble) GL_FALSE);
-    gluTessBeginPolygon(ZnTesselator.tess, &cv->tristrip);
+    gluTessProperty(wi->tess, GLU_TESS_BOUNDARY_ONLY, (GLdouble) GL_FALSE);
+    gluTessBeginPolygon(wi->tess, cv);
     /*
      * We need to take care of the final (after transformation) winding
      * direction of the polygon in order to have the right tesselation
@@ -520,7 +641,7 @@ UpdateTristrip(CurveItem	cv,
      */
     if (!revert) {
       for (j = 0; j < poly->num_contours; j++){
-	gluTessBeginContour(ZnTesselator.tess);
+	gluTessBeginContour(wi->tess);
 	/*printf("Dbut contour %d num_points %d\n",
 	  j, poly->contours[j].num_points);*/
 	for (k = 0; k < poly->contours[j].num_points; k++) {
@@ -528,15 +649,15 @@ UpdateTristrip(CurveItem	cv,
 	  v[0] = poly->contours[j].points[k].x;
 	  v[1] = poly->contours[j].points[k].y;
 	  v[2] = 0;
-	  gluTessVertex(ZnTesselator.tess, v, &poly->contours[j].points[k]);
+	  gluTessVertex(wi->tess, v, &poly->contours[j].points[k]);
 	}
 	/*printf("\n");*/
-	gluTessEndContour(ZnTesselator.tess);
+	gluTessEndContour(wi->tess);
       }
     }
     else {
       for (j = 0; j < poly->num_contours; j++){
-	gluTessBeginContour(ZnTesselator.tess);
+	gluTessBeginContour(wi->tess);
 	/*printf("revert Dbut contour %d num_points %d\n",
 	  j, poly->contours[j].num_points);*/
 	for (i = (int) (poly->contours[j].num_points-1); i >= 0; i--) {
@@ -544,20 +665,20 @@ UpdateTristrip(CurveItem	cv,
 	  v[0] = poly->contours[j].points[i].x;
 	  v[1] = poly->contours[j].points[i].y;
 	  v[2] = 0;
-	  gluTessVertex(ZnTesselator.tess, v, &poly->contours[j].points[i]);
+	  gluTessVertex(wi->tess, v, &poly->contours[j].points[i]);
 	}
 	/*printf("\n");*/
-	gluTessEndContour(ZnTesselator.tess);
+	gluTessEndContour(wi->tess);
       }
     }
-    gluTessEndPolygon(ZnTesselator.tess);
-    cdata = ZnTesselator.combine_list;
+    gluTessEndPolygon(wi->tess);
+    cdata = wi->tess_combine_list;
     while (cdata) {
       cnext = cdata->next;
       ZnFree(cdata);
       cdata = cnext;
     }
-    ZnTesselator.combine_list = NULL;
+    wi->tess_combine_list = NULL;
   }
   /*printf("Fin UpdateTristrips sur %d\n", ((ZnItem) cv)->id);*/
 }
@@ -567,17 +688,29 @@ UpdateOutlines(CurveItem	cv,
 	       ZnPoly		*poly,
 	       ZnBool		revert)
 {
+  ZnWInfo	*wi = ((ZnItem) cv)->wi;
   ZnCombineData	*cdata, *cnext;
   GLdouble	v[3];
   unsigned int	j, k;
   int		i;
 
   /*printf("UpdateOutlines sur %d\n", ((ZnItem) cv)->id);*/
-  gluTessProperty(ZnTesselator.tess, GLU_TESS_WINDING_RULE, (GLdouble) cv->fill_rule);
+  gluTessCallback(wi->tess, GLU_TESS_BEGIN_DATA,
+		  (_GLUfuncptr) CurveTessBegin);
+  gluTessCallback(wi->tess, GLU_TESS_VERTEX_DATA,
+		  (_GLUfuncptr) CurveTessVertex);
+  gluTessCallback(wi->tess, GLU_TESS_END_DATA,
+		  (_GLUfuncptr) CurveTessEnd);
+  gluTessCallback(wi->tess, GLU_TESS_COMBINE_DATA,
+		  (_GLUfuncptr) CurveTessCombine);
+  gluTessCallback(wi->tess, GLU_TESS_ERROR_DATA,
+		  (_GLUfuncptr) CurveTessError);
+  gluTessProperty(wi->tess, GLU_TESS_WINDING_RULE, (GLdouble) cv->fill_rule);
+  gluTessNormal(wi->tess, 0.0, 0.0, -1.0);
 
   if (cv->outlines.num_contours == 0) {
-    gluTessProperty(ZnTesselator.tess, GLU_TESS_BOUNDARY_ONLY, (GLdouble) GL_TRUE);
-    gluTessBeginPolygon(ZnTesselator.tess, &cv->outlines);
+    gluTessProperty(wi->tess, GLU_TESS_BOUNDARY_ONLY, (GLdouble) GL_TRUE);
+    gluTessBeginPolygon(wi->tess, cv);
 
     /*
      * We need to take care of the final (after transformation) winding
@@ -586,36 +719,36 @@ UpdateOutlines(CurveItem	cv,
      */
     if (!revert) {
       for (j = 0; j < poly->num_contours; j++){
-	gluTessBeginContour(ZnTesselator.tess);
+	gluTessBeginContour(wi->tess);
 	for (k = 0; k < poly->contours[j].num_points; k++) {
 	  v[0] = poly->contours[j].points[k].x;
 	  v[1] = poly->contours[j].points[k].y;
 	  v[2] = 0;
-	  gluTessVertex(ZnTesselator.tess, v, &poly->contours[j].points[k]);
+	  gluTessVertex(wi->tess, v, &poly->contours[j].points[k]);
 	}
-	gluTessEndContour(ZnTesselator.tess);
+	gluTessEndContour(wi->tess);
       }
     }
     else {
       for (j = 0; j < poly->num_contours; j++){
-	gluTessBeginContour(ZnTesselator.tess);
+	gluTessBeginContour(wi->tess);
 	for (i = (int) (poly->contours[j].num_points-1); i >= 0; i--) {
 	  v[0] = poly->contours[j].points[i].x;
 	  v[1] = poly->contours[j].points[i].y;
 	  v[2] = 0;
-	  gluTessVertex(ZnTesselator.tess, v, &poly->contours[j].points[i]);
+	  gluTessVertex(wi->tess, v, &poly->contours[j].points[i]);
 	}
-	gluTessEndContour(ZnTesselator.tess);
+	gluTessEndContour(wi->tess);
       }
     }
-    gluTessEndPolygon(ZnTesselator.tess);
-    cdata = ZnTesselator.combine_list;
+    gluTessEndPolygon(wi->tess);
+    cdata = wi->tess_combine_list;
     while (cdata) {
       cnext = cdata->next;
       ZnFree(cdata);
       cdata = cnext;
     }
-    ZnTesselator.combine_list = NULL;
+    wi->tess_combine_list = NULL;
   }
   /*printf("Fin UpdateOutlines sur %d\n", ((ZnItem) cv)->id);*/
 }
@@ -700,8 +833,8 @@ ComputeCoordinates(ZnItem	item,
      */
     if (c1->controls) {
       segment_start = 0;
-      ZnListEmpty(ZnWorkPoints);
-      ZnListAdd(ZnWorkPoints, &c2->points[0], ZnListTail);
+      ZnListEmpty(wi->work_pts);
+      ZnListAdd(wi->work_pts, &c2->points[0], ZnListTail);
       /*printf("moveto %g@%g\n", c2->points[0].x, c2->points[0].y);*/
       for (j = 1; j < c1->num_points; j++) {
 	if (!c1->controls[j]) {
@@ -713,11 +846,11 @@ ComputeCoordinates(ZnItem	item,
 		   c2->points[j].x, c2->points[j].y);*/
 	    ZnGetBezierPoints(&c2->points[segment_start],
 			      &c2->points[segment_start+1], &c2->points[j-1],
-			      &c2->points[j], ZnWorkPoints, 0.5);
+			      &c2->points[j], wi->work_pts, 0.5);
 	  }
 	  else {
 	    /*printf("lineto %g@%g\n", c2->points[j].x, c2->points[j].y);*/
-	    ZnListAdd(ZnWorkPoints, &c2->points[j], ZnListTail);
+	    ZnListAdd(wi->work_pts, &c2->points[j], ZnListTail);
 	  }
 	  segment_start = j;
 	}
@@ -730,19 +863,19 @@ ComputeCoordinates(ZnItem	item,
       if (c1->controls[c1->num_points-1]) {
 	ZnGetBezierPoints(&c2->points[segment_start],
 			  &c2->points[segment_start+1], &c2->points[c1->num_points-1],
-			  &c2->points[0], ZnWorkPoints, 0.5);
+			  &c2->points[0], wi->work_pts, 0.5);
       }
 
       /*
        * Replace the original path by the expanded, closing it as
        * needed (one open contour).
        */
-      num_points =ZnListSize(ZnWorkPoints);
+      num_points =ZnListSize(wi->work_pts);
       if (c2->num_points != c1->num_points) {
 	num_points++;
       }
       c2->points = ZnRealloc(c2->points, num_points*sizeof(ZnPoint));
-      memcpy(c2->points, ZnListArray(ZnWorkPoints), num_points*sizeof(ZnPoint));
+      memcpy(c2->points, ZnListArray(wi->work_pts), num_points*sizeof(ZnPoint));
       if (c2->num_points != c1->num_points) {
 	c2->points[num_points-1] = c2->points[0];
       }
@@ -1057,7 +1190,7 @@ Draw(ZnItem	item)
   ZnWInfo	*wi = item->wi;
   CurveItem	cv = (CurveItem) item;
   XGCValues 	values;
-  unsigned int	i, j, num_points=0;
+  unsigned int	i, j, num_points=0, num2;
   unsigned int	gc_mask;
   ZnPoint	*points=NULL;
   XPoint	*xpoints=NULL;
@@ -1078,14 +1211,14 @@ Draw(ZnItem	item)
     if (cv->tile != ZnUnspecifiedImage) {
       if (!ZnImageIsBitmap(cv->tile)) { /* Fill tiled */
 	values.fill_style = FillTiled;
-	values.tile = ZnImagePixmap(cv->tile, wi->win);
+	values.tile = ZnImagePixmap(cv->tile);
 	values.ts_x_origin = ZnNearestInt(item->item_bounding_box.orig.x);
 	values.ts_y_origin = ZnNearestInt(item->item_bounding_box.orig.y);
 	gc_mask |= GCTileStipXOrigin|GCTileStipYOrigin|GCTile;
       }
       else { /* Fill stippled */
 	values.fill_style = FillStippled;
-	values.stipple = ZnImagePixmap(cv->tile, wi->win);
+	values.stipple = ZnImagePixmap(cv->tile);
 	values.ts_x_origin = ZnNearestInt(item->item_bounding_box.orig.x);
 	values.ts_y_origin = ZnNearestInt(item->item_bounding_box.orig.y);
 	gc_mask |= GCTileStipXOrigin|GCTileStipYOrigin|GCStipple|GCForeground;
@@ -1116,8 +1249,8 @@ Draw(ZnItem	item)
 	}
       }
       else {
-	ZnListAssertSize(ZnWorkXPoints, num_points);
-	xpoints = ZnListArray(ZnWorkXPoints);
+	ZnListAssertSize(wi->work_xpts, num_points);
+	xpoints = ZnListArray(wi->work_xpts);
 	for (j = 0; j < num_points; j++) {
 	  xpoints[j].x = ZnNearestInt(points[j].x);
 	  xpoints[j].y = ZnNearestInt(points[j].y);
@@ -1165,20 +1298,23 @@ Draw(ZnItem	item)
       }
       else {
 	values.fill_style = FillStippled;
-	values.stipple = ZnImagePixmap(cv->line_pattern, wi->win);
+	values.stipple = ZnImagePixmap(cv->line_pattern);
 	XChangeGC(wi->dpy, wi->gc,
 		  GCFillStyle|GCStipple|GCLineWidth|GCJoinStyle|GCCapStyle|GCForeground,
 		  &values);
       }
       for (j = 0; j < cv->outlines.num_contours; j++) {
-	num_points = cv->outlines.contours[j].num_points;
+	num2 = num_points = cv->outlines.contours[j].num_points;
 	points = cv->outlines.contours[j].points;
-	ZnListAssertSize(ZnWorkXPoints, num_points);
-	xpoints = ZnListArray(ZnWorkXPoints);
-	for (i = 0; i < num_points; i++) {
+	ZnListAssertSize(wi->work_xpts, num_points);
+	xpoints = ZnListArray(wi->work_xpts);
+	for (i = 0; i < num2; i++) {
 	  xpoints[i].x = ZnNearestInt(points[i].x);
 	  xpoints[i].y = ZnNearestInt(points[i].y);
 	}
+	if (ISSET(cv->flags, CLOSED_BIT)) {
+	  xpoints[num2] = xpoints[0];
+	}	
 	XDrawLines(wi->dpy, wi->draw_buffer, wi->gc,
 		   xpoints, (int) num_points, CoordModeOrigin);
       }
@@ -1217,14 +1353,14 @@ Draw(ZnItem	item)
     h_width = (width+1)/2;
     h_height = (height+1)/2;
     values.fill_style = FillStippled;
-    values.stipple = ZnImagePixmap(cv->marker, wi->win);
+    values.stipple = ZnImagePixmap(cv->marker);
     values.foreground = ZnGetGradientPixel(cv->marker_color, 0.0);
     XChangeGC(wi->dpy, wi->gc, GCFillStyle|GCStipple|GCForeground, &values);
     for (j = 0; j < cv->outlines.num_contours; j++) {
       num_points = cv->outlines.contours[j].num_points;
       points = cv->outlines.contours[j].points;
-      ZnListAssertSize(ZnWorkXPoints, num_points);
-      xpoints = (XPoint *) ZnListArray(ZnWorkXPoints);
+      ZnListAssertSize(wi->work_xpts, num_points);
+      xpoints = (XPoint *) ZnListArray(wi->work_xpts);
       for (i = 0; i < num_points; i++) {
 	xpoints[i].x = (short) ZnNearestInt(points[i].x);
 	xpoints[i].y = (short) ZnNearestInt(points[i].y);
@@ -1590,8 +1726,8 @@ Pick(ZnItem	item,
  **********************************************************************************
  */
 static void
-PostScript(ZnItem	item __unused,
-	   ZnBool	prepass __unused)
+PostScript(ZnItem		item __unused,
+	   ZnPostScriptInfo	ps_info __unused)
 {
 }
 
@@ -1706,15 +1842,11 @@ Coords(ZnItem		item,
 	if (c->points) {
 	  ZnFree(c->points); 
 	}
-	c->points = ZnMalloc(*num_pts*sizeof(ZnPoint));
+	c->points = (ZnPoint *) ZnMalloc(*num_pts*sizeof(ZnPoint));
 	c->num_points = *num_pts;
 	memcpy(c->points, *pts, *num_pts*sizeof(ZnPoint));
-	if (c->controls) {
-	  ZnFree(c->controls);
-	  c->controls = NULL;
-	}
 	if (*controls) {
-	  c->controls = ZnMalloc(*num_pts*sizeof(char));
+	  c->controls = ZnRealloc(c->controls, *num_pts*sizeof(char));
 	  memcpy(c->controls, *controls, *num_pts*sizeof(char));
 	}
       }
@@ -2087,9 +2219,9 @@ PickVertex(ZnItem	item,
        */
       if (i == (unsigned int) *contour) {
 	j = (*vertex+1) % num_points;
-	new_dist = ZnLineToPointDist(&points[*vertex], &points[j], &po, NULL);
+	new_dist = ZnLineToPointDist(&points[*vertex], &points[j], &po);
 	k = ((unsigned int)(*vertex-1)) % num_points;
-	dist2 = ZnLineToPointDist(&points[*vertex], &points[k], &po, NULL);
+	dist2 = ZnLineToPointDist(&points[*vertex], &points[k], &po);
 	if (dist2 < new_dist) {
 	  *o_vertex = k;
 	}
@@ -2130,12 +2262,11 @@ GetAnchor(ZnItem	item,
  **********************************************************************************
  */
 static ZnItemClassStruct CURVE_ITEM_CLASS = {
-  "curve",
   sizeof(CurveItemStruct),
-  cv_attrs,
   0,			/* num_parts */
-  0,			/* flags */
-  -1,
+  False,		/* has_anchors */
+  "curve",
+  cv_attrs,
   Init,
   Clone,
   Destroy,
