@@ -4,7 +4,7 @@
  * Authors		: Patrick Lecoanet.
  * Creation date	: 
  *
- * $Id: Field.c,v 1.23 2004/05/19 13:04:54 lecoanet Exp $
+ * $Id: Field.c,v 1.18 2003/10/02 12:03:00 lecoanet Exp $
  */
 
 /*
@@ -38,7 +38,7 @@
 #include <stdlib.h>
 
 
-static const char rcsid[] = "$Id: Field.c,v 1.23 2004/05/19 13:04:54 lecoanet Exp $";
+static const char rcsid[] = "$Id: Field.c,v 1.18 2003/10/02 12:03:00 lecoanet Exp $";
 static const char compile_id[]="$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";
 
 
@@ -155,14 +155,12 @@ ComputeFieldAttachment(ZnFieldSet	field_set,
   Field		fptr;
   Tk_FontMetrics fm;
 
-  /*printf("ComputeFieldAttachment in\n");*/
   fptr = &field_set->fields[field];
   if (ISSET(fptr->flags, CACHE_OK)) {
     field_bbox->orig.x = (ZnPos) fptr->orig_x;
     field_bbox->orig.y = (ZnPos) fptr->orig_y;
     field_bbox->corner.x = fptr->corner_x;
     field_bbox->corner.y = fptr->corner_y;
-    /*printf("ComputeFieldAttachment in cache\n");*/
     return;
   }
 
@@ -223,7 +221,7 @@ ComputeFieldAttachment(ZnFieldSet	field_set,
   case ZN_LF_DIM_LABEL:
     {
       ZnDim	lh;
-
+      
       GetLabelBBox(field_set, &real_width, &lh);
       break;
     }
@@ -401,8 +399,6 @@ ComputeFieldAttachment(ZnFieldSet	field_set,
   fptr->corner_x = (short) field_bbox->corner.x;
   fptr->corner_y = (short) field_bbox->corner.y;
   SET(fptr->flags, CACHE_OK);
-
-  /*printf("ComputeFieldAttachment out\n");*/
 }
 
 
@@ -498,18 +494,15 @@ GetLabelBBox(ZnFieldSet	field_set,
   unsigned int	i, num_fields;
   ZnDim		clip_w, clip_h;
   
-  /*printf("GetLabelBBox in\n");*/
   if ((field_set->label_width >= 0.0) && (field_set->label_height >= 0.0)) {
     *w = field_set->label_width;
     *h = field_set->label_height;
-    /*printf("GetLabelBBox in cache\n");*/
     return;
   }
 
   lf = field_set->label_format;
   if (lf == NULL) {
     *w = *h = field_set->label_width = field_set->label_height = 0.0;
-    /*printf("GetLabelBBox no labelformat\n");*/
     return;
   }
 
@@ -557,7 +550,6 @@ GetFieldBBox(ZnFieldSet		field_set,
 {
   ZnReal	ox, oy;
   
-  /*printf("GetFieldBBox in\n");*/
   if (field_set->label_format) {
     ox = ZnNearestInt(field_set->label_pos.x);
     oy = ZnNearestInt(field_set->label_pos.y);
@@ -570,7 +562,6 @@ GetFieldBBox(ZnFieldSet		field_set,
   else {
     ZnResetBBox(field_bbox);
   }
-  /*printf("GetFieldBBox out\n");*/
 }
 
 
@@ -839,7 +830,7 @@ InitFields(ZnFieldSet	field_set)
     field->tile = ZnUnspecifiedImage;
     field->font = Tk_GetFont(wi->interp, wi->win, Tk_NameOfFont(wi->font));
 #ifdef GL
-    field->tfi = ZnGetTexFont(wi, field->font);
+    field->tfi = NULL;
 #endif
     field->border_edges = ZN_NO_BORDER;
     field->alignment = TK_JUSTIFY_LEFT;
@@ -901,7 +892,7 @@ CloneFields(ZnFieldSet	field_set)
     }
     field->font = Tk_GetFont(wi->interp, wi->win, Tk_NameOfFont(field->font));
 #ifdef GL
-    field->tfi = ZnGetTexFont(wi, field->font);
+    field->tfi = NULL;
 #endif
     field->color = ZnGetGradientByValue(field->color);
     field->fill_color = ZnGetGradientByValue(field->fill_color);
@@ -992,7 +983,7 @@ ConfigureField(ZnFieldSet	fs,
   if (old_font != fptr->font) {
     if (fptr->tfi) {
       ZnFreeTexFont(fptr->tfi);
-      fptr->tfi = ZnGetTexFont(wi, fptr->font);
+      fptr->tfi = NULL;
     }
   }
 #endif
@@ -1055,7 +1046,7 @@ QueryField(ZnFieldSet		fs,
     return TCL_ERROR;
   }
     
-  if (ZnQueryAttribute(fs->item->wi->interp, &fs->fields[field], field_attrs,
+  if (ZnQueryAttribute(fs->item->wi, &fs->fields[field], field_attrs,
 		       argv[0]) == TCL_ERROR) {
     return TCL_ERROR;
   }
@@ -1793,7 +1784,7 @@ DrawField(ZnWInfo	*wi,
     if (fptr->tile != ZnUnspecifiedImage) {
       if (!ZnImageIsBitmap(fptr->tile)) { /* Fill tiled */
 	values.fill_style = FillTiled;
-	values.tile = ZnImagePixmap(fptr->tile, wi->win);
+	values.tile = ZnImagePixmap(fptr->tile);
 	values.ts_x_origin = (int) bbox->orig.x;
 	values.ts_y_origin = (int) bbox->orig.y;
 	XChangeGC(wi->dpy, wi->gc,
@@ -1802,7 +1793,7 @@ DrawField(ZnWInfo	*wi,
       }
       else { /* Fill stippled */
 	values.fill_style = FillStippled;
-	values.stipple = ZnImagePixmap(fptr->tile, wi->win);
+	values.stipple = ZnImagePixmap(fptr->tile);
 	values.ts_x_origin = (int) bbox->orig.x;
 	values.ts_y_origin = (int) bbox->orig.y;
 	XChangeGC(wi->dpy, wi->gc,
@@ -1833,7 +1824,7 @@ DrawField(ZnWInfo	*wi,
 	fw = ZnNearestInt(bbox->corner.x - bbox->orig.x);
 	fh = ZnNearestInt(bbox->corner.y - bbox->orig.y);
 
-	pixmap = ZnImagePixmap(fptr->image, wi->win);
+	pixmap = ZnImagePixmap(fptr->image);
 	photo_region = ZnImageRegion(fptr->image);
 	ZnCurrentClip(wi, &clip_region, NULL, &simple);
 	clip = TkCreateRegion();
@@ -2049,6 +2040,11 @@ RenderField(ZnWInfo	*wi,
        */
       num_bytes = strlen(fptr->text);
       if (num_bytes) {
+	if (!fptr->tfi) {
+	  if (! (fptr->tfi = ZnGetTexFont(wi, fptr->font))) {
+	    continue;
+	  }
+	}
 	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 	if (sel_start >= 0) {
 	  color = ZnGetGradientColor(ti->sel_color, 0.0, &alpha);
@@ -2113,7 +2109,7 @@ RenderField(ZnWInfo	*wi,
     color = ZnGetGradientColor(fptr->border_color, 0.0, &alpha);
     alpha = ZnComposeAlpha(alpha, wi->alpha);
     glColor4us(color->red, color->green, color->blue, alpha);
-    glLineWidth(1.0);
+    glLineWidth(1.5);
     ZnSetLineStyle(wi, ZN_LINE_SIMPLE);
     glBegin(GL_LINES);
     if (fptr->border_edges & ZN_LEFT_BORDER) {
