@@ -4,7 +4,7 @@
  * Authors		: Patrick Lecoanet.
  * Creation date	:
  *
- * $Id: Map.c,v 1.58 2004/04/30 14:30:39 lecoanet Exp $
+ * $Id: Map.c,v 1.53 2003/10/02 12:03:43 lecoanet Exp $
  */
 
 /*
@@ -40,7 +40,7 @@
 #include <stdio.h>
 
 
-static const char rcsid[] = "$Id: Map.c,v 1.58 2004/04/30 14:30:39 lecoanet Exp $";
+static const char rcsid[] = "$Id: Map.c,v 1.53 2003/10/02 12:03:43 lecoanet Exp $";
 static const char compile_id[]="$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";
 
 
@@ -440,7 +440,7 @@ Query(ZnItem		item,
       int		argc __unused,
       Tcl_Obj *CONST	argv[])
 {
-  if (ZnQueryAttribute(item->wi->interp, item, map_attrs, argv[0]) == TCL_ERROR) {
+  if (ZnQueryAttribute(item->wi, item, map_attrs, argv[0]) == TCL_ERROR) {
     return TCL_ERROR;
   }  
 
@@ -464,6 +464,7 @@ ComputeCoordinates(ZnItem	item,
   MapItem		map = (MapItem) item;
   ZnMapInfoId		map_info;
   ZnMapInfoLineStyle	line_style;
+  int			line_width;
   ZnMapInfoTextStyle	text_style;
   char			symbol;
   char			*text;
@@ -485,11 +486,10 @@ ComputeCoordinates(ZnItem	item,
   ZnPoint		tmp_from, tmp_to;
   XArc			arc;
   ZnBBox		bbox, bbox_inter, zn_bbox;
-  ZnPos			x_from_w, y_from_w, x_to_w, y_to_w;
-  ZnPos			start_angle, extend;
-  ZnDim			radius_w, line_width;
-  int			radius;
-  ZnPoint		*new_marks;
+  int			x_from_w, y_from_w, x_to_w, y_to_w;
+  int			radius, start_angle, extend;
+  int			radius_w;
+  ZnMapInfoPoint		new_marks;
   unsigned int		n_new_marks;
   Tk_Font		text_font;
   int			sym_w2=0, sym_h2=0;
@@ -803,8 +803,8 @@ ComputeCoordinates(ZnItem	item,
     arc.y	= (int) (center.y - radius);
     arc.width	= 2 * radius;
     arc.height	= arc.width;
-    arc.angle1	= ((unsigned short) start_angle) * 64;
-    arc.angle2	= ((unsigned short) extend) * 64;
+    arc.angle1	= start_angle*64;
+    arc.angle2	= extend*64;
     
     switch (line_style) {
     case ZnMapInfoLineSimple:
@@ -952,17 +952,17 @@ ComputeCoordinates(ZnItem	item,
   ZnListAssertSize(map->marks, num_marks);
   
   ZnAddPointsToBBox(&item->item_bounding_box,
-		    ZnListArray(map->vectors), ZnListSize(map->vectors));
+		    (ZnPoint *) ZnListArray(map->vectors),
+		    ZnListSize(map->vectors));
   ZnAddPointsToBBox(&item->item_bounding_box,
-		    ZnListArray(map->dashed_vectors), ZnListSize(map->dashed_vectors));
+		    (ZnPoint *) ZnListArray(map->dashed_vectors),
+		    ZnListSize(map->dashed_vectors));
   ZnAddPointsToBBox(&item->item_bounding_box,
-		    ZnListArray(map->dotted_vectors), ZnListSize(map->dotted_vectors));
+		    (ZnPoint *) ZnListArray(map->dotted_vectors),
+		    ZnListSize(map->dotted_vectors));
   ZnAddPointsToBBox(&item->item_bounding_box,
-		    ZnListArray(map->mixed_vectors), ZnListSize(map->mixed_vectors));
-  item->item_bounding_box.orig.x -= 0.5;
-  item->item_bounding_box.orig.y -= 0.5;
-  item->item_bounding_box.corner.x += 0.5;
-  item->item_bounding_box.corner.y += 0.5;
+		    (ZnPoint *) ZnListArray(map->mixed_vectors),
+		    ZnListSize(map->mixed_vectors));
 }
 
 
@@ -1005,7 +1005,6 @@ Draw(ZnItem	item)
   char		tmp_str[] = ".";
   XGCValues	values;
   unsigned int	i, cnt;
-  ZnDim		line_width_w;
   int		line_width;
 
   if (map->map_info == NULL) {
@@ -1024,15 +1023,15 @@ Draw(ZnItem	item)
       }
       else { /* Fill stippled */
 	values.fill_style = FillStippled;
-	values.stipple = ZnImagePixmap(map->fill_pattern, wi->win);
+	values.stipple = ZnImagePixmap(map->fill_pattern);
 	XChangeGC(wi->dpy, wi->gc,
 		  GCFillStyle | GCStipple | GCForeground, &values);
       }
       
       cnt = ZnListSize(map->vectors);
       if (cnt) {
-	ZnListAssertSize(ZnWorkXPoints, cnt);
-	xpoints = (XPoint *) ZnListArray(ZnWorkXPoints);
+	ZnListAssertSize(wi->work_xpts, cnt);
+	xpoints = (XPoint *) ZnListArray(wi->work_xpts);
 	points = (ZnPoint *) ZnListArray(map->vectors);
 	for (i = 0; i < cnt; i++) {
 	  xpoints[i].x = (int) points[i].x;
@@ -1071,16 +1070,12 @@ Draw(ZnItem	item)
       points = (ZnPoint *) ZnListArray(map->vectors);
       for (i = 0; i < cnt; i += 2) {
 	if (ZnLineInBBox(&points[i], &points[i+1], &wi->damaged_area) >= 0) {
-          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width_w, NULL,
+          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width, NULL,
 			 NULL, NULL, NULL);
-	  line_width = (int) line_width_w;
           if (line_width != values.line_width) {
             values.line_width = line_width;
             XChangeGC(wi->dpy, wi->gc, GCLineWidth, &values);
           }
-	  /*printf("Dessin d'une ligne de %d %d  %d %d\n",
-		 (int)points[i].x, (int)points[i].y,
-		 (int)points[i+1].x, (int)points[i+1].y);*/
 	  XDrawLine(wi->dpy, wi->draw_buffer, wi->gc,
 		    (int) points[i].x,
 		    (int) points[i].y,
@@ -1104,8 +1099,7 @@ Draw(ZnItem	item)
       points = (ZnPoint *) ZnListArray(map->dashed_vectors);
       for (i = 0; i < cnt; i += 2) {
 	if (ZnLineInBBox(&points[i], &points[i+1], &wi->damaged_area) >= 0) {
-          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width_w, NULL, NULL, NULL, NULL);
-	  line_width = (int) line_width_w;	  
+          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width, NULL, NULL, NULL, NULL);
           if (line_width != values.line_width) {
             values.line_width = line_width;
             XChangeGC(wi->dpy, wi->gc, GCLineWidth, &values);
@@ -1134,8 +1128,7 @@ Draw(ZnItem	item)
       points = (ZnPoint *) ZnListArray(map->dotted_vectors);
       for (i = 0; i < cnt; i += 2) {
 	if (ZnLineInBBox(&points[i], &points[i+1], &wi->damaged_area) >= 0) {
-          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width_w, NULL, NULL, NULL, NULL);
-	  line_width = (int) line_width_w;
+          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width, NULL, NULL, NULL, NULL);
           if (line_width != values.line_width) {
             values.line_width = line_width;
             XChangeGC(wi->dpy, wi->gc, GCLineWidth, &values);
@@ -1163,8 +1156,7 @@ Draw(ZnItem	item)
       points = (ZnPoint *) ZnListArray(map->mixed_vectors);
       for (i = 0; i < cnt; i += 2) {
 	if (ZnLineInBBox(&points[i], &points[i+1], &wi->damaged_area) >= 0) {
-          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width_w, NULL, NULL, NULL, NULL);
-	  line_width = (int) line_width_w;
+          ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &line_width, NULL, NULL, NULL, NULL);
           if (line_width != values.line_width) {
             values.line_width = line_width;
             XChangeGC(wi->dpy, wi->gc, GCLineWidth, &values);
@@ -1278,7 +1270,7 @@ Draw(ZnItem	item)
 	    ZnSizeOfImage(sym, &w ,&h);
 	    ox = ((int) points[i].x) - w/2;
 	    oy = ((int) points[i].y) - h/2;
-	    values.stipple = ZnImagePixmap(sym, wi->win);
+	    values.stipple = ZnImagePixmap(sym);
 	    values.ts_x_origin = ox;
 	    values.ts_y_origin = oy;
 	    XChangeGC(wi->dpy, wi->gc,
@@ -1292,7 +1284,7 @@ Draw(ZnItem	item)
 	ZnSizeOfImage(wi->map_distance_symbol, &w, &h);
 	cnt = ZnListSize(map->marks);
 	points = ZnListArray(map->marks);
-	values.stipple = ZnImagePixmap(wi->map_distance_symbol, wi->win);
+	values.stipple = ZnImagePixmap(wi->map_distance_symbol);
 	XChangeGC(wi->dpy, wi->gc, GCStipple, &values);
 	for (i = 0; i < cnt; i++) {
 	  ox = ((int) points[i].x) - w/2;
@@ -1326,10 +1318,8 @@ Render(ZnItem	item)
   char		*text;
   char		tmp_str[] = ".";
   unsigned int	i, cnt;
-  int		w, h;
   XColor	*color;
-  GLfloat	line_width;
-  ZnDim		new_width;
+  int		line_width, new_width, w, h;
   unsigned short alpha;
 
   if (!map->map_info) {
@@ -1354,8 +1344,8 @@ Render(ZnItem	item)
   }
   else { /* Not filled */
     if (ZnListSize(map->vectors)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       ZnSetLineStyle(wi, ZN_LINE_SIMPLE);
       cnt = ZnListSize(map->vectors);
       points = ZnListArray(map->vectors);
@@ -1365,8 +1355,8 @@ Render(ZnItem	item)
           ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &new_width, NULL,
 			 NULL, NULL, NULL);
           if (new_width != line_width) {
-	    line_width = (GLfloat)new_width;
-	    glLineWidth(line_width);
+	    line_width = new_width;
+	    glLineWidth((GLfloat) line_width);
           }
 	  glVertex2d(points[i].x, points[i].y);
 	  glVertex2d(points[i+1].x, points[i+1].y);
@@ -1375,8 +1365,8 @@ Render(ZnItem	item)
       glEnd();
     }
     if (ZnListSize(map->dashed_vectors)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       ZnSetLineStyle(wi, ZN_LINE_DASHED);
       cnt = ZnListSize(map->dashed_vectors);
       points = ZnListArray(map->dashed_vectors);
@@ -1386,8 +1376,8 @@ Render(ZnItem	item)
           ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &new_width, NULL,
 			 NULL, NULL, NULL);
           if (new_width != line_width) {
-	    line_width = (GLfloat)new_width;
-	    glLineWidth(line_width);
+	    line_width = new_width;
+	    glLineWidth((GLfloat) line_width);
           }
 	  glVertex2d(points[i].x, points[i].y);
 	  glVertex2d(points[i+1].x, points[i+1].y);
@@ -1397,8 +1387,8 @@ Render(ZnItem	item)
       glDisable(GL_LINE_STIPPLE);
     }
     if (ZnListSize(map->dotted_vectors)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       ZnSetLineStyle(wi, ZN_LINE_DOTTED);
       cnt = ZnListSize(map->dotted_vectors);
       points = ZnListArray(map->dotted_vectors);
@@ -1408,8 +1398,8 @@ Render(ZnItem	item)
           ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &new_width, NULL,
 			 NULL, NULL, NULL);
           if (new_width != line_width) {
-	    line_width = (GLfloat)new_width;
-	    glLineWidth(line_width);
+	    line_width = new_width;
+	    glLineWidth((GLfloat) line_width);
           }
 	  glVertex2d(points[i].x, points[i].y);
 	  glVertex2d(points[i+1].x, points[i+1].y);
@@ -1419,8 +1409,8 @@ Render(ZnItem	item)
       glDisable(GL_LINE_STIPPLE);
     }
     if (ZnListSize(map->mixed_vectors)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       ZnSetLineStyle(wi, ZN_LINE_MIXED);
       cnt = ZnListSize(map->mixed_vectors);
       points = ZnListArray(map->mixed_vectors);
@@ -1430,8 +1420,8 @@ Render(ZnItem	item)
           ZnMapInfoGetLine(map_info, i/2, NULL, NULL, &new_width, NULL,
 			 NULL, NULL, NULL);
           if (new_width != line_width) {
-	    line_width = (GLfloat)new_width;
-	    glLineWidth(line_width);
+	    line_width = new_width;
+	    glLineWidth((GLfloat) line_width);
           }
 	  glVertex2d(points[i].x, points[i].y);
 	  glVertex2d(points[i+1].x, points[i+1].y);
@@ -1442,26 +1432,26 @@ Render(ZnItem	item)
     }
 
     if (ZnListSize(map->arcs)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
     }
     if (ZnListSize(map->dashed_arcs)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       glLineStipple(1, 0xF0F0);
       glEnable(GL_LINE_STIPPLE);
       glDisable(GL_LINE_STIPPLE);
     }
     if (ZnListSize(map->dotted_arcs)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       glLineStipple(1, 0x18C3);
       glEnable(GL_LINE_STIPPLE);
       glDisable(GL_LINE_STIPPLE);
     }
     if (ZnListSize(map->mixed_arcs)) {
-      line_width = 1.0;
-      glLineWidth(line_width);
+      line_width = 1;
+      glLineWidth((GLfloat) line_width);
       glLineStipple(1, 0x27FF);
       glEnable(GL_LINE_STIPPLE);
       glDisable(GL_LINE_STIPPLE);
@@ -1489,7 +1479,7 @@ Render(ZnItem	item)
 
     if (map->symbol_patterns) {
       ZnImage sym, *syms = ZnListArray(map->symbol_patterns);
-      int     num_syms = ZnListSize(map->symbol_patterns);
+      int     w, h, num_syms = ZnListSize(map->symbol_patterns);
       
       cnt = ZnListSize(map->symbols);
       points = ZnListArray(map->symbols);
@@ -1561,6 +1551,29 @@ Pick(ZnItem	item __unused,
 }
 
 
+/*
+ **********************************************************************************
+ *
+ * Coords --
+ *	Nothing to do for maps (or too complex anyway).
+ *
+ **********************************************************************************
+ */
+static int
+Coords(ZnItem		item,
+       int		contour __unused,
+       int		index __unused,
+       int		cmd __unused,
+       ZnPoint		**pts __unused,
+       char		**controls __unused,
+       unsigned int	*num_pts __unused)
+{
+  Tcl_AppendResult(item->wi->interp,
+		   " maps doesn't support the coords command", NULL);
+  return TCL_ERROR;
+}
+
+
 /*
  **********************************************************************************
  *
@@ -1569,8 +1582,8 @@ Pick(ZnItem	item __unused,
  **********************************************************************************
  */
 static void
-PostScript(ZnItem	item __unused,
-	   ZnBool	prepass __unused)
+PostScript(ZnItem		item __unused,
+	   ZnPostScriptInfo	ps_info __unused)
 {
 }
 
@@ -1584,12 +1597,11 @@ PostScript(ZnItem	item __unused,
  */
 
 static ZnItemClassStruct MAP_ITEM_CLASS = {
-  "map",
   sizeof(MapItemStruct),
-  map_attrs,
   0,			/* num_parts */
-  0,			/* flags */
-  -1,
+  False,		/* has_anchors */
+  "map",
+  map_attrs,
   Init,
   Clone,
   Destroy,
@@ -1599,7 +1611,7 @@ static ZnItemClassStruct MAP_ITEM_CLASS = {
   NULL,			/* GetAnchor */
   NULL,			/* GetClipVertices */
   NULL,			/* GetContours */
-  NULL,
+  Coords,
   NULL,			/* InsertChars */
   NULL,			/* DeleteChars */
   NULL,			/* Cursor */
